/*******************************************************************************
 * Copyright (c) 2003-2005, Michael Bartl
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the license in the file License.txt
 * 
 * Created on 11.06.2004
 *******************************************************************************/

package com.mbartl.viplugineclipse;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

import org.eclipse.jface.text.IDocument;
import org.eclipse.jface.text.ITextViewer;
import org.eclipse.jface.text.ITextViewerExtension;
import org.eclipse.swt.custom.StyledText;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.part.MultiPageEditorPart;
import org.eclipse.ui.texteditor.AbstractTextEditor;
import org.eclipse.ui.texteditor.ITextEditor;

import com.mbartl.viimplementation.layer.LayerInformation;
import com.mbartl.viimplementation.layer.ViLayer;
import com.mbartl.viimplementation.layer.ViLayerManager;

/**
 * @author Michael Bartl
 */
public class ActivateLayer {
	private static boolean _isIBMSSEEditor = false;

	public static void activateLayer(IWorkbenchPart part) {
		ITextEditor editor = checkAndGetEditor(part);
		if (editor == null) {
			return;
		}
		if (layerExists(editor)) {
			IDocument document = editor.getDocumentProvider().getDocument(
					editor.getEditorInput());
			ViLayer layer = ViLayerManager.getInstance().getLayerById(
					editor.toString());
			// layer exists and document is the same
			if (document.get().equals(layer.getDocument().get())) {
				ViPluginPlugin.getDefault().log(
						"activated layer: " + editor.toString());
				ViLayerManager.getInstance().activateLayer(editor.toString());
			}
			// layer exists but new document
			else {
				ViPluginPlugin.getDefault().log(
						"reuse layer: " + editor.toString());
				ViLayerManager.getInstance().removeViLayer(editor.toString());
				createNewLayer(editor);
			}
		} else if (!editor.isEditable()) {
			ViPluginPlugin.getDefault().log(
					"not creating new layer because editor is read-only: " + editor.toString());
		} else {
			ViPluginPlugin.getDefault().log(
					"creating new layer: " + editor.toString());
			createNewLayer(editor);
		}

	}

	private static void createNewLayer(ITextEditor editor) {
		Method me;
		try {
			me = AbstractTextEditor.class.getDeclaredMethod("getSourceViewer",
					null);
			me.setAccessible(true);
			Object viewer = me.invoke(editor, null);

			ITextViewer textViewer = (ITextViewer) viewer;
			StyledText styledText = textViewer.getTextWidget();

			EditorImplementation editorImplementation = new EditorImplementation(
					editor, textViewer);
			IDocument document = editor.getDocumentProvider().getDocument(
					editor.getEditorInput());
			ViStatusLineManager slm = new ViStatusLineManager(editor
					.getEditorSite().getActionBars().getStatusLineManager());

			DocumentAdapter documentAdapter = new DocumentAdapter(document);

			LayerInformation info = new LayerInformation(editorImplementation,
					documentAdapter);
			info.setStatusLineManager(slm);
			if (!_isIBMSSEEditor) {
				info.setUndoManager(new ViUndoManager(editorImplementation,
						textViewer));
				info.setCaret(new ViCaret(styledText));
			}
			info.setBookmark(new BookMark(editor));
			info.setMessageDialog(new ViMessageDialog(ViPluginPlugin
					.getDefault().getWorkbench().getActiveWorkbenchWindow()));

			ViLayer layer = ViLayerManager.getInstance().createLayer(
					editor.toString(), info);

			documentAdapter.setLayer(layer);
			layer.setKeyListener(new TheVerifyKeyListener(layer,
					(ITextViewerExtension) viewer));
		} catch (SecurityException e) {
			e.printStackTrace();
		} catch (NoSuchMethodException e) {
			e.printStackTrace();
		} catch (IllegalArgumentException e) {
			e.printStackTrace();
		} catch (IllegalAccessException e) {
			e.printStackTrace();
		} catch (InvocationTargetException e) {
			e.printStackTrace();
		}
	}

	/**
	 * Check if the given workbenchpart implements an ITextEditor interface
	 * 
	 * @param part
	 * @return
	 */
	private static ITextEditor checkAndGetEditor(IWorkbenchPart part) {
		_isIBMSSEEditor = false;
		if (part == null) {
			return null;
		}
		Object editor = null;
		// MultiPageEditors have to be handled first, because sometimes they are
		// instances of ITextEditor too
		if (part instanceof MultiPageEditorPart) {
			try {
				Method me = MultiPageEditorPart.class.getDeclaredMethod(
						"getActiveEditor", null);
				me.setAccessible(true);
				Object multiPageEditorPartActivePage = me.invoke(part, null);
				if (multiPageEditorPartActivePage instanceof ITextEditor) {
					editor = (ITextEditor) multiPageEditorPartActivePage;
					_isIBMSSEEditor = multiPageEditorPartActivePage.getClass()
							.getName().startsWith("com.ibm.sse.editor");
				}
			} catch (SecurityException e) {
				e.printStackTrace();
			} catch (NoSuchMethodException e) {
				e.printStackTrace();
			} catch (IllegalArgumentException e) {
				e.printStackTrace();
			} catch (IllegalAccessException e) {
				e.printStackTrace();
			} catch (InvocationTargetException e) {
				e.printStackTrace();
			}
		}
		// a single page ITextEditor
		else if (part instanceof ITextEditor) {
			editor = (ITextEditor) part;
		}
		// maybe a multipage editor that can adapt?
		else {
			editor = part.getAdapter(ITextEditor.class);
		}
		return (ITextEditor) editor;
	}

	/**
	 * @param editor
	 * @return true if the given editor already has a viLayer
	 */
	private static boolean layerExists(ITextEditor editor) {
		return ViLayerManager.getInstance().getLayerById(editor.toString()) != null;
	}
}