/*******************************************************************************
 * Copyright (c) 2003-2004, Michael Bartl
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the license in the file License.txt
 * 
 * Created on 28.06.2004
 *******************************************************************************/

package com.mbartl.viplugineclipse;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IMarker;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IFileEditorInput;
import org.eclipse.ui.texteditor.ITextEditor;

import com.mbartl.viimplementation.interfaces.IBookmark;

/**
 * This class is the connection to the eclipse bookmark system. You can add or
 * goTo bookmarks with the respective functions.
 * 
 * @author Michael Bartl
 */
public class BookMark implements IBookmark {

	private final ITextEditor _editor;
	
	/**
	 * @param editor
	 */
	public BookMark(ITextEditor editor) {
		_editor = editor;
	}

	public void add(String name, int linePos) {
		try {
			IMarker marker = getMarker(name, true);
			marker.setAttribute(IMarker.LINE_NUMBER, linePos + 1);
			marker.setAttribute(IMarker.MESSAGE, name);
		} catch (CoreException e) {
			e.printStackTrace();
		}
	}

	public void remove(String arg0) {
		// not needed yet
	}

	private IMarker getMarker(String name, boolean create) {
		IEditorInput editorInput = _editor.getEditorInput();
		if (editorInput instanceof IFileEditorInput == false) {
			return null;
		}

		IFileEditorInput fileEditorInput = (IFileEditorInput) editorInput;
		IFile file = fileEditorInput.getFile();

		try {
			// get all Bookmark Markers
			IMarker[] markers = file.findMarkers(IMarker.BOOKMARK, true,
					IResource.DEPTH_INFINITE);

			// search if it exists already
			for (int i = 0; i < markers.length; i++) {
				if (markers[i].getAttribute(IMarker.MESSAGE, "--").compareTo(
						name) == 0) {
					return markers[i];
				}
			}

			// nothing found => if create == true create a new one
			if (create) {
				return file.createMarker(IMarker.BOOKMARK);
			}
		} catch (CoreException e) {
			e.printStackTrace();
		}
		return null;
	}
}