/*******************************************************************************
 * Copyright (c) 2003-2004, Michael Bartl
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the license in the file License.txt
 * 
 * Created on 10.06.2004
 *******************************************************************************/

package com.mbartl.viplugineclipse;

import org.eclipse.core.runtime.Platform;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IPartListener;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.IWorkbenchWindowActionDelegate;

import com.mbartl.viimplementation.layer.ViLayerManager;
import com.mbartl.viimplementation.license.CheckLicenseFile;

/**
 * This Action is associated via the plugin.xml file with the Toolbar Icon, Menu
 * Item and keyboard shortcut to activate/deactivate the viPlugin
 * 
 * @author Michael Bartl
 */
public class LoadViAction implements IWorkbenchWindowActionDelegate {

	private boolean _activated = false;

	private ViWindowListener _windowListener = new ViWindowListener();

	private IPartListener _partListener = new ViPartListener();

	public void run(IAction action) {
		IWorkbenchWindow window = ViPluginPlugin.getDefault().getWorkbench()
				.getActiveWorkbenchWindow();
		load(window);
	}

	public void load(IWorkbenchWindow window) {
		ViLayerManager.getInstance()
				.setEditorManager(new EditorManager(window));
		try {
			String[] licenseLocations = new String[2];
			licenseLocations[0] = Platform.getInstallLocation().getURL()
					.getPath();
			licenseLocations[1] = Platform.getInstanceLocation().getURL()
					.getPath();
			boolean fullVersion = CheckLicenseFile
					.checkLicenseLocations(licenseLocations);
			ViLayerManager.getInstance().setFullVersion(fullVersion);
		} catch (Exception e) {
			MessageDialog.openInformation(window.getShell(),
					"Error with license", e.getMessage());
		}

		if (!_activated) {
			ViPluginPlugin.getDefault().getWorkbench().addWindowListener(
					_windowListener);
			window.getPartService().addPartListener(_partListener);
			IEditorPart editor = window.getActivePage().getActiveEditor();
			ActivateLayer.activateLayer(editor);
		} else {
			ViLayerManager.getInstance().removeAllLayers();
			window.getPartService().removePartListener(_partListener);
			ViPluginPlugin.getDefault().getWorkbench().removeWindowListener(
					_windowListener);
			IEditorPart editor = window.getActivePage().getActiveEditor();
			if (editor != null) {
				editor.getEditorSite().getActionBars().getStatusLineManager()
						.setMessage("viPlugin unloaded!");
			}
		}
		_activated = !_activated;
	}
	
	public void setActivated(boolean activated) {
		_activated = activated;
	}

	/**
	 * Selection in the workbench has been changed. We can change the state of
	 * the 'real' action here if we want, but this can only happen after the
	 * delegate has been created.
	 * 
	 * @see IWorkbenchWindowActionDelegate#selectionChanged
	 */
	public void selectionChanged(IAction action, ISelection selection) {
		// not used by ViPlugin
	}

	/**
	 * We can use this method to dispose of any system resources we previously
	 * allocated.
	 * 
	 * @see IWorkbenchWindowActionDelegate#dispose
	 */
	public void dispose() {
		// not used by ViPlugin
	}

	public void init(IWorkbenchWindow window) {
		// not used by ViPlugin
	}
}