/*******************************************************************************
 * Copyright (c) 2003-2004, Michael Bartl
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the license in the file License.txt
 * 
 * Created on 10.06.2004
 *******************************************************************************/

package com.mbartl.viplugineclipse;

import java.awt.event.KeyEvent;

import org.eclipse.jface.text.ITextViewerExtension;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.VerifyKeyListener;
import org.eclipse.swt.events.VerifyEvent;

import com.mbartl.viimplementation.interfaces.IKeyListener;
import com.mbartl.viimplementation.layer.ViLayer;
import com.mbartl.viimplementation.util.ViKeyEvent;

public class TheVerifyKeyListener implements VerifyKeyListener, IKeyListener {

    private ViLayer _layer;

    private ITextViewerExtension _extension;
    
    /**
     * @param extension
     * @param _layer
     */
    public TheVerifyKeyListener(ViLayer layer, ITextViewerExtension extension) {
        _layer = layer;
        _extension = extension;
    }

    public void verifyKey(VerifyEvent e) {
        if (e.keyCode == SWT.ARROW_UP) {
            e.keyCode = KeyEvent.VK_UP;
        } else if (e.keyCode == SWT.ARROW_DOWN) {
            e.keyCode = KeyEvent.VK_DOWN;
        } else if (e.keyCode == SWT.ARROW_LEFT) {
            e.keyCode = KeyEvent.VK_LEFT;
        } else if (e.keyCode == SWT.ARROW_RIGHT) {
            e.keyCode = KeyEvent.VK_RIGHT;
        } else if (e.keyCode == 13) {
            e.keyCode = 0;
            e.character = '\n';
        }
        // fix for mantis bug #65 --> ' has the same keyCode as VK_RIGHT
        if (e.character == '\'') {
        	e.keyCode = 0;
        }
        // fix for mantis bug #60 --> HOME and END key doesn't work
        if (e.keyCode == SWT.HOME) {
            e.keyCode = KeyEvent.VK_HOME;
        }
        if (e.keyCode == SWT.END) {
            e.keyCode = KeyEvent.VK_END;
        }
        
        ViKeyEvent keyEvent = new ViKeyEvent(e.character, e.keyCode,
                e.stateMask);
        if (_layer.verifyKeyEvent(keyEvent)) {
            e.doit = false;
        }
    }
    
    public void register() {
        _extension.prependVerifyKeyListener(this);
    }

    public void unregister() {
        _extension.removeVerifyKeyListener(this);
    }
}