/*******************************************************************************
 * Copyright (c) 2003-2004, Michael Bartl
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the license in the file License.txt
 * 
 * Created on 14.05.2004
 *******************************************************************************/

package com.mbartl.viplugineclipse;

import org.eclipse.swt.custom.StyledText;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.widgets.Caret;

import com.mbartl.viimplementation.interfaces.ICaret;

/**
 * @author Michael Bartl
 */
public class ViCaret implements ICaret {

	private final StyledText _textWidget;

	private final Caret _caret;

	private final Point _originalSize;

	private static final int CARET_THIN = 1;

	/**
	 * create VIM style caret
	 */
	public ViCaret(StyledText textWidget) {
		_caret = textWidget.getCaret();
		_originalSize = _caret.getSize();
		_caret.setImage(null);
		_textWidget = textWidget;
	}

	private void setCaretWidth(int width) {
		if (!_caret.isDisposed()) {
			_caret.setSize(width, _caret.getSize().y);
		}
	}

	public void setCaretThin() {
		setCaretWidth(CARET_THIN);
	}

	public void setCaretThick() {
		if (!_caret.isDisposed()) {
			GC gc = new GC(_textWidget);
			setCaretWidth(gc.getFontMetrics().getAverageCharWidth());
			gc.dispose();
		}
	}

	public void restoreOriginalCaret() {
		if (!_caret.isDisposed()) {
			_caret.setSize(_originalSize);
		}
	}
}