/*******************************************************************************
 * Copyright (c) 2003-2004, Michael Bartl
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the license in the file License.txt
 * 
 * Created on 10.06.2004
 *******************************************************************************/

package com.mbartl.viplugineclipse;

import java.io.IOException;
import java.util.MissingResourceException;
import java.util.ResourceBundle;

import nanoxml.XMLParseException;

import org.eclipse.core.resources.IWorkspace;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Platform;
import org.eclipse.core.runtime.Status;
import org.eclipse.ui.plugin.AbstractUIPlugin;
import org.osgi.framework.BundleContext;

import com.mbartl.viimplementation.preferences.ViPreferenceService;

/**
 * The main plugin class to be used in the desktop
 * 
 * @author Michael Bartl
 */
public class ViPluginPlugin extends AbstractUIPlugin {

	//The shared instance.
	private static ViPluginPlugin plugin;

	//Resource bundle.
	private ResourceBundle resourceBundle;

	private final boolean debug = false;

	/**
	 * The constructor.
	 */
	public ViPluginPlugin() {
		super();
		plugin = this;
		try {
			resourceBundle = ResourceBundle
					.getBundle("viPlugin.viPluginPluginResources");
		} catch (MissingResourceException x) {
			resourceBundle = null;
		}
	}

	/**
	 * This method is called upon plug-in activation
	 */
	public void start(BundleContext context) throws Exception {
		super.start(context);
		// get preferences file
		String[] preferencesLocations = new String[2];
		preferencesLocations[0] = Platform.getInstallLocation().getURL()
				.getPath();
		preferencesLocations[1] = Platform.getInstanceLocation().getURL()
				.getPath();
		ViPreferenceService service = ViPreferenceService.getInstance();
		try {
			service.getPreferences().checkPreferencesLocations(
					preferencesLocations);
		} catch (XMLParseException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	/**
	 * This method is called when the plug-in is stopped
	 */
	public void stop(BundleContext context) throws Exception {
		super.stop(context);
	}

	/**
	 * Returns the shared instance.
	 */
	public static ViPluginPlugin getDefault() {
		return plugin;
	}

	/**
	 * Returns the workspace instance.
	 */
	public static IWorkspace getWorkspace() {
		return ResourcesPlugin.getWorkspace();
	}

	/**
	 * Returns the string from the plugin's resource bundle, or 'key' if not
	 * found.
	 */
	public static String getResourceString(String key) {
		ResourceBundle bundle = ViPluginPlugin.getDefault().getResourceBundle();
		try {
			return (bundle != null) ? bundle.getString(key) : key;
		} catch (MissingResourceException e) {
			return key;
		}
	}

	/**
	 * Returns the plugin's resource bundle,
	 */
	public ResourceBundle getResourceBundle() {
		return resourceBundle;
	}

	public void log(String message) {
		if (debug) {
			Status status = new Status(IStatus.INFO, "viPlugin", IStatus.OK,
					message, null);
			Platform.getLog(getBundle()).log(status);
		}
	}
}