/*******************************************************************************
 * Copyright (c) 2003-2004, Michael Bartl
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the license in the file License.txt
 * 
 * Created on 11.06.2004
 *******************************************************************************/

package com.mbartl.viplugineclipse;

import org.eclipse.jface.text.DefaultUndoManager;
import org.eclipse.jface.text.ITextViewer;

import com.mbartl.viimplementation.interfaces.IUndoManager;
import com.mbartl.viimplementation.preferences.Preferences;
import com.mbartl.viimplementation.preferences.ViPreferenceService;

/**
 * @author Michael Bartl
 */
public class ViUndoManager implements IUndoManager {

    private DefaultUndoManager _undoManager;

    private ITextViewer _viewer;

    private EditorImplementation _editor;

    public ViUndoManager(EditorImplementation editor, ITextViewer viewer) {
        _viewer = viewer;
        _editor = editor;
        int levels = ViPreferenceService.getInstance().getInt(
                Preferences.UNDOLEVELS);
        _undoManager = new DefaultUndoManager(levels);
        viewer.setUndoManager(_undoManager);
        _undoManager.connect(_viewer);
    }

    public void redo() {
        _undoManager.redo();
        _editor.setCaretPosition(_editor.getSelection().getOffset());
    }

    public void undo() {
        _undoManager.undo();
        _editor.setCaretPosition(_editor.getSelection().getOffset());
    }

    public void beginCompoundChange() {
        _undoManager.beginCompoundChange();
    }

    public void endCompoundChange() {
        _undoManager.endCompoundChange();
    }

    public void setUndoLevels(int levels) {
        _undoManager.setMaximalUndoLevel(levels);
    }

    public boolean canUndo() {
        return _undoManager.undoable();
    }
}
